@@ -31,9 +31,7 @@
  *   You should also use DOMAIN\\USERNAME as described in README.MSCHAP80
  */
 
-#ifndef lint
-static char rcsid[] = "$Id: chap_ms.c,v 1.13 1999/03/16 03:15:14 paulus Exp $";
-#endif
+#define RCSID	"$Id: chap_ms.c,v 1.15 1999/08/13 06:46:12 paulus Exp $"
 
 #ifdef CHAPMS
 
@@ -57,6 +55,8 @@ static char rcsid[] = "$Id: chap_ms.c,v 1.13 1999/03/16 03:15:14 paulus Exp $";
 #include <des.h>
 #endif
 
+static const char rcsid[] = RCSID;
+
 typedef struct {
     u_char LANManResp[24];
     u_char NTResp[24];
