@@ -17,7 +17,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#define RCSID	"$Id: upap.c,v 1.23 1999/11/20 05:11:47 paulus Exp $"
+#define RCSID	"$Id: upap.c,v 1.24 2001/03/08 05:11:16 paulus Exp $"
 
 /*
  * TODO:
@@ -38,15 +38,17 @@ static bool hide_password = 1;
  */
 static option_t pap_option_list[] = {
     { "hide-password", o_bool, &hide_password,
-      "Don't output passwords to log", 1 },
+      "Don't output passwords to log", OPT_PRIO | 1 },
     { "show-password", o_bool, &hide_password,
-      "Show password string in debug log messages", 0 },
+      "Show password string in debug log messages", OPT_PRIOSUB | 0 },
+
     { "pap-restart", o_int, &upap[0].us_timeouttime,
-      "Set retransmit timeout for PAP" },
+      "Set retransmit timeout for PAP", OPT_PRIO },
     { "pap-max-authreq", o_int, &upap[0].us_maxtransmits,
-      "Set max number of transmissions for auth-reqs" },
+      "Set max number of transmissions for auth-reqs", OPT_PRIO },
     { "pap-timeout", o_int, &upap[0].us_reqtimeout,
-      "Set time limit for peer PAP authentication" },
+      "Set time limit for peer PAP authentication", OPT_PRIO },
+
     { NULL }
 };
 
